#!/usr/bin/perl -w 

# Copyright (C) 2000 Julian T. J. Midgley <jtjm@xenoclast.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# A copy of version 2 of the GNU General Public License may be found
# in the file "LICENCE" in the original Agnostos installation driectory.


use taskman_lib;
use strict;
use CGI;

# Parameters passed:
# table : tablename
# id    : id within the table whose entries are to be displayed



#--------------------------------------------------------------------------
# dispview ($table, $id, $show_cancelled, $sortby)
#
# displays the view for the requested 'user'.  $table should be set to
# one of 'workgroup', 'user', or 'department'.

sub
dispview 
{
    my ($table, $id, $show_cancelled, $sortby) = @_;
    $show_cancelled = 'false' unless (defined $show_cancelled);
    my $assid = gen_assigned_name($table);
    my $dbh1 = dbserver_connect('agnostos');
    my ($dbq1, $todoid, $title, $priority, $status, $numrows, $sql_query, %users, %groups, %depts);
    my ($userid, $groupid, $depid, $date, $row, $td, $overdue);
    my $tablehead="#efea9e";         # Background colour of headings
    my $tableheadcol="#ffffff";      # Text colour of headings
    my @col_row=("#f1f3e9","#dfd7c9");
    my @col_inprog=("#78c653","#9ee27f");
    my @col_overdue=("#f40e0e","#e03e3e");
    my @col_done=("#90ccf9","#5ab1f4",);
    #my @col_queued=("#03b206","#58d310");
    #my @col_inprog=@col_row;
    my @col_queued=@col_row;
    
    %users  = fetch_ents( 'user',       $dbh1);
    %groups = fetch_ents( 'workgroup',  $dbh1);
    %depts  = fetch_ents( 'department', $dbh1);
    
    $sql_query = "SELECT todo.todoid, title, priority, status, 
                         assigned.userid, assigned.groupid, assigned.depid, date, 
                         (TO_DAYS(date)-TO_DAYS(CURDATE())) 
                     FROM todo, assigned 
                     WHERE todo.todoid = assigned.todoid 
                     AND assigned.$assid = $id ";
    unless ($show_cancelled eq "true") {
	$sql_query .=  "AND status != 'cancelled' AND status != 'done' ";
    }
    if ($sortby eq "status") {
	$sql_query .= " ORDER BY status,priority,date";
    }
    elsif ($sortby eq "priority") {
	$sql_query .= " ORDER BY priority,status,date";
    }
    elsif ($sortby eq "date") {
	$sql_query .= " ORDER BY date,status,priority";
    }
    else {
	$sql_query .= " ORDER BY status,priority,date";
    }
    $dbq1 = $dbh1 ->prepare($sql_query);
    $numrows = $dbq1 -> execute;
    if ( $numrows > 0 ) {
	$td = "<td bgcolor=$tablehead>";
	print "<table><tr>$td<b>ID</b>$td<b>Task</b>\n";
	print "$td<b><a href=\"view_tasks.pl?sortby=priority\">Priority</a></b>\n";
	print "$td<b><a href=\"view_tasks.pl?sortby=status\">Status</a></b>\n";
	print "$td<b>User</b>$td<b>Group</b>$td<b>Department</b>\n";
	print "$td<b><a href=\"view_tasks.pl?sortby=date\">Due Date</a></b>\n";
	$row = 0;
	while ( ($todoid, $title, $priority, $status, $userid, $groupid, $depid, $date, $overdue) = $dbq1 -> fetchrow_array ) {
	    $row = $row ? 0 : 1 ;
	    if ($status eq 'in-progress') {
		$td = $col_inprog[$row];
	    }
	    elsif ($status eq 'queued') {
		$td = $col_queued[$row];
	    }
	    elsif ($status eq 'done' or $status eq 'cancelled') {
		$td = $col_done[$row];
	    }
	    else {
		$td = $col_row[$row];
	    }
	    if (defined $overdue && $overdue < 0) {
		$td = $col_overdue[$row];
	    }
	    $td = "<td bgcolor=$td>";
	    print "<tr>$td<a href=\"edit_task.pl?todoid=$todoid\">$todoid";
	    print "$td$title$td$priority$td$status\n";
	    if (defined $date && $date =~ /(\d+)-(\d+)-(\d+)/) {
		$date = "$3-$2-$1";
	    }
	    else {
		$date = "";
	    }
	    
	    print "$td$users{$userid}$td$groups{$groupid}$td$depts{$depid}$td$date\n";
	}
	print "</table>\n";
    }
    $dbq1 -> finish;
    $dbh1 -> disconnect;
    return;
}

#--------------------------------------------------------------------------
# check_setview
#
# Reads the parameters passed to the CGI and returns appropriate
# $table and $id settings.

sub
check_setview 
{
    my $query = new CGI;
    my $user       = $query -> param('user');
    my $workgroup  = $query -> param('workgroup');
    my $dept       = $query -> param('department');
    my $orig_table = $query -> param('table');
    my $orig_id    = $query -> param('id');
    my ( $res_table, $res_id );
    my $found = 0;

    if (defined $user) {
	unless ($user =~ /_1$/) {
	    $user =~ /(.*)_(.*)/;
	    ( $res_table, $res_id ) = ( $1, $2 );
	    $found = 1;
	}
    }
    if (defined $workgroup) {
	unless ($workgroup =~ /_1$/) {
	    $workgroup =~ /(.*)_(.*)/;
	    ( $res_table, $res_id ) = ( $1, $2 );
	    $found = 1;
	}
    }
    if (defined $dept) {
	unless ($dept =~ /_1$/) {
	    $dept =~ /(.*)_(.*)/;
	    ( $res_table, $res_id ) = ( $1, $2 );
	$found = 1;
	}
    }
    if ($found) {
	return ($res_table, $res_id);
    }
    else {
	return ($orig_table, $orig_id);
    }
}


#--------------------------------------------------------------------------
# main

my $query = new CGI;
my $dbh   = dbserver_connect('agnostos');
#my $table = $query -> param('table');
#my $id    = $query -> param('id');
my ($dbq, $ent_name, $table, $id, @cookie);
my $show_cancelled = $query -> param('cancelled');
my $sortby         = $query -> param('sortby');
if (!defined $sortby ) {
    $sortby = 'status';
}
($table,$id) = check_setview();
if ( (! defined $table) or (! defined $id) or $table eq "" ) {
    $table = $query -> cookie ("INTSYS_AGNOSTOS_TABLE");
    $id = $query -> cookie("INTSYS_AGNOSTOS_ID");
}
else {
    push ( @cookie, $query -> cookie (-name    => 'INTSYS_AGNOSTOS_TABLE',
				      -expires => '+1y',
				      -value   => "$table"));
    push ( @cookie, $query -> cookie (-name    => 'INTSYS_AGNOSTOS_ID',
				      -expires => '+1y',
				      -value   => "$id"));

}
if ( ! defined $table ) {
    print html_header("No view selected",$query);
    print "<h1>Error - no view selected</h1>".
	"You must select a view from the front page.<p>  Once you have done so, a cookie will".
	" be set, and you can then use the &quot;View Tasks&quot; link.";
    print html_footer;
    $dbh -> disconnect;
    exit 0;
}
my ($act_id, $act_name) = get_table($table);


$dbq = $dbh -> prepare("SELECT $act_name FROM $table WHERE $act_id = $id");
$dbq -> execute;
$ent_name = $dbq -> fetchrow_array;
$dbq -> finish;

print html_header("Tasklist for $ent_name", $query, @cookie);
print "<h2>Tasklist for $ent_name</h2><p>";

print "<table><tr><td><a href=\"edit_task.pl\">Add new entry</a></td><td>&nbsp;|&nbsp;</td>".
      "<td><a href=\"view_tasks.pl?cancelled=true\">Show Done/Cancelled Items</a></td></tr></table>\n";
dispview($table, $id, $show_cancelled, $sortby);


print html_footer;

$dbh -> disconnect;


